SuperStrict
Include "openal.bmx"
Include "flashlight.bmx"
GCSetMode 2
SetGraphicsDriver GLMax2DDriver()
Const sx% = 800, sy% = 600
Graphics sx, sy
SeedRnd MilliSecs()

Global water#[5,2]
Global waterState# = 0.0
Global speedFactor# = 1.2, timeFactor# = 1.0, lightFactor# = 1.0, waterReversed% = False
Global foundHorns% = 0
Const START_HORNS% = 3
Global timeReset% = 0, lightReset% = 0, waterReset% = 0
Const TIME_TIME% = 4000, LIGHT_TIME% = 10000, WATER_TIME% = 700
Global startTime%
Const FAST_TIME% = 20000
Global bg:TImage, playerImage:TImage, angle#, radius#, staying% = True
Global x#, y#, dir#
Global timer:TTimer
Const MIN_ANGLE# = 1
Const MAX_ANGLE# = 80
Const TOTAL_LIGHT# =6000
Const ROTATION# = 3
Global infoTime% = -1
Const START_INFO_TIME% = 250, FADE_INFO_TIME% = 50
Const DISPLAY_TIME% = 250, DISPLAY_FADE_TIME% = 50
Const FAST_WIN% = 20000
Global displayAchivements:TList = New TList
Global level% = -1, levelFactor# = 0.0, levelFactors#[] = [.5, 1.0, 2.0]
ShowMenu
Function Init()
  OpenALInit
  LoadHorn
  LoadAchivements
  LoadDatabase
  THorn.image = LoadImage("horn.png")
  TClock.image = LoadImage("clock.png")
  TLight.image = LoadImage("light.png")
  TShoes.image = LoadImage("shoes.png")
  TWind.image = LoadImage("wind.png")
  bg = LoadImage("bg.png")
  MidHandleImage THorn.image
  playerImage = LoadImage("player.png")
  SetImageHandle playerImage, 5, 0
  timer = CreateTimer(60)
  Reset
End Function
Function Play%(l%)
  level = l
  levelFactor = levelFactors[l-1]
  StartTime = MilliSecs()
  Local frames% = 0
  Repeat
    If frames < 10 Then
      frames :+ 1
    EndIf
    WaitTimer timer
    Cls
    SetBlend ALPHABLEND
    If level = 1 Then
      SetAlpha .3
      DrawSunset
      SetAlpha 1
    EndIf
    SetColor 255, 255, 255
    Local flash#[] = ComputeFlashlight(x, y, dir, angle, radius, .6)
    DrawLight flash
    SetAlpha .6
    DrawOval x-20, y-20, 40, 40
    SetAlpha 1
    SetBlend SHADEBLEND
    TileImage bg
    For Local item:TItem = EachIn TItem.list
      If PointInWater(item.x, item.y) Then
        item.Destroy
      Else
        item.Draw 0, 0
      EndIf
    Next
    SetColor 0, 0, 255
    SetBlend SHADEBLEND
    DrawWater
    SetBlend ALPHABLEND
    SetRotation 180-dir
    SetAlpha 1
    SetColor 255, 255, 255
    DrawImage playerImage, x, y
    SetRotation 0
    If infoTime > 0 Then
      If infoTime < FADE_INFO_TIME Then
        SetAlpha infoTime/Float(FADE_INFO_TIME)
        DrawText "Sieg. Druecke [Esc] zum Beenden", 0, 0
        SetAlpha 1
      Else
        DrawText "Sieg. Druecke [Esc] zum Beenden", 0, 0
      EndIf
      infoTime :- 1
    EndIf
?Debug
    DrawText "speedFactor: "+speedFactor, 0, 20
    DrawText "timeFactor: "+timeFactor, 0, 40
?
    ShowDisplayAchivements
    Flip 0
    If KeyDown(KEY_UP) Then
      x :+ Sin(dir)*speedFactor*timeFactor
      y :+ Cos(dir)*speedFactor*timeFactor
      If x < 0 Or x > sx Or y < 0 Or y > sy Then TAchivement.Unlock ACHIVEMENT_ESCAPE
      x = Max(0, Min(sx, x))
      y = Max(0, Min(sy, y))
      SetListenerPos x, y
      SetListenerVector Sin(dir)*speedFactor*timeFactor, Cos(dir)*speedFactor*timeFactor
      For Local item:TItem = EachIn TItem.list
        If Abs(item.x-x) < 10 And Abs(item.y-y) < 10 Then item.Collect
      Next
      staying = False
    ElseIf Not staying Then
      SetListenerVector 0, 0
      staying = True
    EndIf
    If PointInWater(x, y) Then
      If Not ShowGameOver() Then Return -1
      Exit
    EndIf
    dir :+ (KeyDown(KEY_LEFT) - KeyDown(KEY_RIGHT))*speedFactor*ROTATION*timeFactor
    If KeyDown(KEY_PAGEDOWN) Then
      angle = Min(angle+1, MAX_ANGLE)
      ComputeRadius
    ElseIf KeyDown(KEY_PAGEUP)
      angle = Max(angle-1, MIN_ANGLE)
      ComputeRadius
    EndIf
    SetListenerDir dir
    If waterReversed Then
      UpdateWater Rnd(-0.001, -0.01)*timeFactor*levelFactor
    Else
      UpdateWater Rnd(0, 0.0003)*timeFactor*levelFactor
      If waterState <= 0.0 Then
        TAchivement.Unlock ACHIVEMENT_BLOWAWAY
        waterReversed = False
        waterReset = 0
        DebugLog "water reset"
      EndIf
    EndIf
    If timeReset <> 0 And timeReset < MilliSecs() Then
      timeFactor = 1.0
      timeReset = 0
      THorn.SetAllSpeed 2.0
      DebugLog "time reset"
    EndIf
    If lightReset <> 0 And lightReset < MilliSecs() Then
      lightFactor = 1.0
      lightReset = 0
      ComputeRadius
      DebugLog "light reset"
    EndIf
    If waterReset <> 0 And waterReset < MilliSecs() Then
      waterReversed = False
      waterReset = 0
      DebugLog "water reset"
    EndIf
    If MouseDown(MOUSE_RIGHT) Then TAchivement.Unlock ACHIVEMENT_CONTEXT
    If KeyHit(KEY_ESCAPE) Then Return foundHorns = START_HORNS
    If KeyHit(KEY_SPACE) Then
      If Not ShowAchivementScreen() Then Return -1
    EndIf
    If AppTerminate() Then Return -1
    GCCollect
  Forever
  Return False
End Function
Function Terminate()
  DestroyAL
  SaveDatabase
  GCCollect
  End
End Function
Function ShowGameOver%()
  TAchivement.Unlock ACHIVEMENT_DIE
  SetColor 255, 255, 255
  Repeat
    WaitTimer timer
    Cls
    DrawText "Game Over", 0, 0
    Flip 0
    If AppTerminate() Then Return 0
    GCCollect
  Until KeyHit(KEY_ESCAPE)
  Return 1
End Function
Function Reset()
  TItem.list.Clear
  x = 0.0
  y = 0.0
  dir = 0.0
  waterState = 0.0
  UpdateWater 0.0
  speedFactor = 1.2
  timeFactor = 1.0
  lightFactor = 1.0
  waterReversed = False
  foundHorns = 0
  timeReset = 0
  lightReset = 0
  waterReset = 0
  infoTime = -1
    angle = 20.0
  ComputeRadius
  THorn.SetAllSpeed 2.0
  staying = True
  SetListenerPos 0.0, 0.0
  SetListenerVector 0.0, 0.0
  SetListenerDir 0.0
  lastHorn = 0
  b4lastHorn = 0
  noOfHorns = 0
  displayAchivements.Clear
  GCCollect
End Function
Function CreateEnvironment()
  For Local i%=1 To 5
    CreateRandomItem
  Next
  For Local i%=1 To START_HORNS
    CreateRandomHorn
  Next
  THorn.SetAllSpeed 2.0
  GCCollect
End Function


Type TItem Abstract
  Global list:TList = New TList
  Field x#, y#
  Method Draw(offsetX#, offsetY#) Abstract
  Method Collect()
  End Method
  Method Destroy()
  End Method
End Type

Function UpdateWater(stp#)
  waterState :+ stp
  Local x# = sx*waterState, y# = sy*waterState
  water[0,0] = x
  water[0,1] = 0
  water[1,0] = x*.9
  water[1,1] = y*.3
  water[2,0] = x*.7
  water[2,1] = y*.7
  water[3,0] = x*.3
  water[3,1] = y*.9
  water[4,0] = 0
  water[4,1] = y
End Function
Function DrawWater()
  Local draw#[12]
  For Local i%=0 Until 5
    draw[i*2] = sx-water[i,0]
    draw[i*2+1] = sy-water[i,1]
  Next
  draw[10] = sx
  draw[11] = sy
  DrawPoly draw
  glBegin GL_TRIANGLE_STRIP
  For Local i%=0 Until 5
    SetColor 0, 0, 255
    glVertex2f sx-water[i,0], sy-water[i,1]
    SetColor 255, 255, 255
    glVertex2f sx-water[i,0]*1.1, sy-water[i,1]*1.1
  Next
  SetAlpha 1
  glEnd
End Function
Function PointInWater%(x#, y#)
  x = sx-x
  y = sy-y
  For Local i%=0 Until 5
    If water[i,0] = x Then Return water[i,1] > y
  Next
  For Local i%=0 Until 4
    If x > water[i,0] Or x < water[i+1,0] Then Continue
    If y > water[i,1] And y > water[i+1,1] Then Continue
    If y < water[i,1] And y < water[i+1,1] Then Return True
    Local m1# = Abs(water[i,1]-water[i+1,1])/Abs(water[i,0]-water[i+1,0])
    Local m2# = Abs(water[i+1,1]-y)/Abs(water[i+1,0]-x)
    If m2 > m1 Then Return True
  Next
  Return False
End Function

Type TClock Extends TItem
  Global image:TImage
  Method Draw(offsetX#, offsetY#)
    DrawImage Self.image, Self.x+offsetX, Self.y+offsetY
  End Method
  Method Collect()
    TAchivement.Unlock ACHIVEMENT_SLOW
    If timeFactor < 1 Then TAchivement.Unlock ACHIVEMENT_SLOW2
    timeFactor :* .5
    timeReset = MilliSecs()+TIME_TIME
    THorn.SetAllSpeed timeFactor*2
    Self.Destroy
  End Method
  Method Destroy()
    Self.list.Remove Self
  End Method
End Type

Type TLight Extends TItem
  Global image:TImage
  Method Draw(offsetX#, offsetY#)
    DrawImage Self.image, Self.x+offsetX, Self.y+offsetY
  End Method
  Method Collect()
    TAchivement.Unlock ACHIVEMENT_BRIGHTER
    If lightFactor > 1 Then TAchivement.Unlock ACHIVEMENT_BRIGHTER2
    lightFactor :* 3
    lightReset = MilliSecs()+LIGHT_TIME
    ComputeRadius
    Self.Destroy
    If Self.list.IsEmpty() Then TAchivement.Unlock ACHIVEMENT_BRIGHT4NOTHING
  End Method
  Method Destroy()
    Self.list.Remove Self
  End Method
End Type

Type TShoes Extends TItem
  Global image:TImage
  Method Draw(offsetX#, offsetY#)
    DrawImage Self.image, Self.x+offsetX, Self.y+offsetY
  End Method
  Method Collect()
    TAchivement.Unlock ACHIVEMENT_FASTER
    If speedFactor > 1.201 Then TAchivement.Unlock ACHIVEMENT_FASTER2
    speedFactor :* 1.1
    Self.Destroy
  End Method
  Method Destroy()
    Self.list.Remove Self
  End Method
End Type
Type TWind Extends TItem
  Global image:TImage
  Method Draw(offsetX#, offsetY#)
    DrawImage Self.image, Self.x+offsetX, Self.y+offsetY
  End Method
  Method Collect()
    TAchivement.Unlock ACHIVEMENT_BLOW
    If foundHorns = 0 Then TAchivement.Unlock ACHIVEMENT_GOODBLOW
    waterReversed = True
    waterReset = MilliSecs()+WATER_TIME
    Self.Destroy
  End Method
  Method Destroy()
    Self.list.Remove Self
  End Method
End Type

Function GetRandomItemCoord(x# Var, y# Var)
  x = Rnd(0, sx-10)
  If x < 20 Then
    y = Rnd(20, sy-10)
  ElseIf x > sx-50
    y = Rnd(0, sy-30)
  Else
    y = Rnd(0, sy-10)
  EndIf
End Function


Function CreateRandomItem()
  Local x#, y#
  GetRandomItemCoord x, y
  Local item:TItem
  Select Rand(1, 4)
   Case 1
    item = New TClock
   Case 2
    item = New TLight
   Case 3
    item = New TShoes
   Case 4
    item = New TWind
   Default
    RuntimeError "Internal error"
  End Select
  item.x = x
  item.y = y
  TItem.list.AddLast item
End Function
Function CreateRandomHorn()
  Local x#, y#
  GetRandomItemCoord x, y
  THorn.Create x, y, 0, 0
End Function
Function ComputeRadius()
  Local half# = angle*.5
  radius = Sqr(TOTAL_LIGHT*lightFactor/(2*Sin(half)*Cos(half)))
End Function
Function DrawBox(x#, y#, w#, h#, range% = 30, border% = 3, dir% = 1)
  Local col%[3], light%[3], dark%[3]
  GetColor col[0], col[1], col[2]
  If border Then
    SetColor 0, 0, 0
    DrawRect x, y, w, h
    x :+ border
    y :+ border
    w :- border*2
    h :- border*2
  EndIf
  For Local i%=0 Until 3
    light[i] = Min(255, col[i]+range)
    dark[i] = Max(0, col[i]-range)
  Next
  glBegin GL_QUADS
  If dir = 1 Then
    SetColor dark[0], dark[1], dark[2]
  Else
    SetColor light[0], light[1], light[2]
  EndIf
  glVertex2f x, y+h 'lb
  glVertex2f x+w, y+h 'rb
  If dir = 1 Then
    SetColor light[0], light[1], light[2]
  Else
    SetColor dark[0], dark[1], dark[2]
  EndIf
  glVertex2f x+w, y 'rt
  glVertex2f x, y 'lt
  glEnd
  SetColor col[0], col[1], col[2]
End Function
Function DrawTextCentered(text$, x#, y#)
  Local lines$[] = text.Split("~n")
  y :- TextHeight(text)*lines.length/2
  For Local line$ = EachIn lines
    DrawText line, x-TextWidth(line)*.5, y
    y :+ TextHeight(line)
  Next
End Function

Type TAchivementGroup Extends TList
  Field col_lock%[3], col_new%[3], col_unlock%[3], bg%[3]
  Method Draw(x#, y#)
    SetColorArray Self.bg
    DrawBox x, y, (TAchivement.BOX_WIDTH+TAchivement.PADDING)*Self.Count()+TAchivement.PADDING,..
                           TAchivement.BOX_HEIGHT+TAchivement.PADDING*2
    x :+ TAchivement.PADDING
    y :+ TAchivement.PADDING
    For Local item:TAchivement = EachIn Self
      If item.locked Then
        SetColorArray Self.col_lock
      ElseIf item.isNew Then
        SetColorArray Self.col_new
      Else
        SetColorArray Self.col_unlock
      EndIf
      item.Draw x, y
      x :+ TAchivement.BOX_WIDTH+TAchivement.PADDING
    Next
  End Method
    '<r>ed/<g>reen/<b>lue | <l>ocked, <n>ew, <u>nlocked, <b>ackground
  Method SetColors(rl%, gl%, bl%, rn%, gn%, bn%, ru%, gu%, bu%, rb%, gb%, bb%)
    Self.col_lock[0] = rl
    Self.col_lock[1] = gl
    Self.col_lock[2] = bl

    Self.col_new[0] = rn
    Self.col_new[1] = gn
    Self.col_new[2] = bn

    Self.col_unlock[0] = ru
    Self.col_unlock[1] = gu
    Self.col_unlock[2] = bu

    Self.bg[0] = rb
    Self.bg[1] = gb
    Self.bg[2] = bb
  End Method
End Type
Const ACHIVEMENT_1HORN% = 0
Const ACHIVEMENT_2HORN% = 1
Const ACHIVEMENT_3HORN% = 2
Const ACHIVEMENT_2HORNS_FAST% = 3
Const ACHIVEMENT_3HORNS_FAST% = 4
Const ACHIVEMENT_HORN_FAIL% = 5

Const ACHIVEMENT_SLOW% = 6
Const ACHIVEMENT_SLOW2% = 7
Const ACHIVEMENT_SLOWHORN% = 8

Const ACHIVEMENT_BRIGHTER% = 9
Const ACHIVEMENT_BRIGHTER2% = 10
Const ACHIVEMENT_BRIGHT4NOTHING% = 11

Const ACHIVEMENT_FASTER% = 12
Const ACHIVEMENT_FASTER2% = 13
Const ACHIVEMENT_DOPPLER% = 14

Const ACHIVEMENT_BLOW% = 15
Const ACHIVEMENT_BLOWAWAY% = 16
Const ACHIVEMENT_GOODBLOW% = 17

Const ACHIVEMENT_DIE% = 18
Const ACHIVEMENT_CRASHER% = 19
Const ACHIVEMENT_CONTEXT% = 20
Const ACHIVEMENT_ESCAPE% = 21
Const ACHIVEMENT_WIN% = 22
Const ACHIVEMENT_WINFAST% = 23
Global achivementsHorn:TAchivementGroup
Global achivementsItem1:TAchivementGroup
Global achivementsItem2:TAchivementGroup
Global achivementsPlayer:TAchivementGroup
Function LoadAchivements()
  achivementsHorn = New TAchivementGroup
  achivementsItem1 = New TAchivementGroup
  achivementsItem2 = New TAchivementGroup
  achivementsPlayer = New TAchivementGroup
  achivementsHorn.SetColors   128, 0,   0,     255, 100, 100,   245, 10,   10,   255, 200, 200   'red theme
  achivementsItem1.SetColors  128, 128, 0,     255, 255, 100,   200, 200, 10,   240, 240, 200   'yellow theme
  achivementsItem2.SetColors  0,   128, 0,     100, 255, 100,   10,   245, 10,   200, 255, 200   'green theme
  achivementsPlayer.SetColors 0,   0,   128,   100, 100, 255,   50,   50,   245, 200, 200, 255   'blue theme
  TAchivement.Create ACHIVEMENT_1HORN, "Sammle ein~nHorn", achivementsHorn
  TAchivement.Create ACHIVEMENT_2HORN, "Sammle zwei~nHoerner", achivementsHorn
  TAchivement.Create ACHIVEMENT_3HORN, "Sammle drei~nHoerner", achivementsHorn
  TAchivement.Create ACHIVEMENT_2HORNS_FAST, "Sammle schnell~nzwei Hoerner", achivementsHorn
  TAchivement.Create ACHIVEMENT_3HORNS_FAST, "Sammle schnell~ndrei Hoerner", achivementsHorn
  TAchivement.Create ACHIVEMENT_HORN_FAIL, "Verfehle ein~nHorn knapp", achivementsHorn

  TAchivement.Create ACHIVEMENT_SLOW, "Verlangsame~ndie Zeit", achivementsItem1
  TAchivement.Create ACHIVEMENT_SLOW2, "Verlangsame~ndie Zeit~ndoppelt", achivementsItem1
  TAchivement.Create ACHIVEMENT_SLOWHORN, "Sammle ein~nlangsames Horn", achivementsItem1
  TAchivement.Create ACHIVEMENT_BRIGHTER, "Mache es~nheller", achivementsItem1
  TAchivement.Create ACHIVEMENT_BRIGHTER2, "Mache es~nnoch heller", achivementsItem1
  TAchivement.Create ACHIVEMENT_BRIGHT4NOTHING, "Mache es~nheller -~nfuer nichts", achivementsItem1

  TAchivement.Create ACHIVEMENT_FASTER, "Gehe schneller", achivementsItem2
  TAchivement.Create ACHIVEMENT_FASTER2, "Gehe noch~nschneller", achivementsItem2
  TAchivement.Create ACHIVEMENT_DOPPLER, "Hoere den~nDoppler-Effekt", achivementsItem2
  TAchivement.Create ACHIVEMENT_BLOW, "Draenge das~nWasser zurueck", achivementsItem2
  TAchivement.Create ACHIVEMENT_BLOWAWAY, "Draenge das~nWasser weg", achivementsItem2
  TAchivement.Create ACHIVEMENT_GOODBLOW, "Gewinne~nwertvolle Zeit", achivementsItem2

  TAchivement.Create ACHIVEMENT_DIE, "STIRB!", achivementsPlayer
  TAchivement.Create ACHIVEMENT_CRASHER, "Bringe das~nProgramm zum~nAbsturz", achivementsPlayer
  TAchivement.Create ACHIVEMENT_CONTEXT, "Es gibt kein~nKontextmenue!", achivementsPlayer
  TAchivement.Create ACHIVEMENT_ESCAPE, "Versuche~nzu entkommen", achivementsPlayer
  TAchivement.Create ACHIVEMENT_WIN, "Gewinne das~nSpiel", achivementsPlayer
  TAchivement.Create ACHIVEMENT_WINFAST, "Gewinne das~nSpiel schnell", achivementsPlayer
End Function




Type TAchivement
  Const NO% = 24
  Const BOX_WIDTH# = 120, BOX_HEIGHT# = 80, PADDING# = 10
  Global array:TAchivement[NO]
  Field locked% = True, isNew%, group:TAchivementGroup '<-- cyclic, but static
  Field text$
  Field displayTime% = 0
  Method Draw(x#, y#)
    DrawBox x, y, Self.BOX_WIDTH, Self.BOX_HEIGHT
    If Self.locked Then Return
    SetColor 0, 0, 0
    DrawTextCentered Self.text, x+Self.BOX_WIDTH/2, y+Self.BOX_HEIGHT/2
  End Method
  Function Create(index%, text$, group:TAchivementGroup)
    Local a:TAchivement = New TAchivement
    a.text = text
    a.group = group
    group.AddLast a
    Self.array[index] = a
  End Function
  Function Unlock(index%)
    If level = 1 Then Return
    Local a:TAchivement = Self.array[index]
    If Not a.locked Then Return
    a.locked = False
    a.isNew  = True
    a.displayTime = DISPLAY_TIME
    displayAchivements.AddLast a
  End Function
  Function Load(bank:TBank)
    Local index%, b%
    For index = 0 Until Self.NO
      If index Mod 8 = 0 Then b = PeekByte(bank, index/8)
      Local bit% = (b Shr (index Mod 8)) & 1
      Self.array[index].locked = bit
    Next
  End Function
  Function Save(bank:TBank)
    Local index%, b%
    For index = 0 Until Self.NO
      b :| Self.array[index].locked Shl (index Mod 8)
      If index Mod 8 = 7 Then
        PokeByte(bank, index/8, b)
        b = 0
      EndIf
    Next
  End Function
End Type
Function SetColorArray(array%[])
  SetColor array[0], array[1], array[2]
End Function
Const DB_FILE$ = "db.dat"
Function LoadDatabase()
  If FileType(DB_FILE) <> 1 Then Return
  Local bank:TBank = LoadBank(DB_FILE)
  TAchivement.Load bank
  Local crashed% = PeekByte(bank, TAchivement.NO/8+1)
  If crashed Then TAchivement.Unlock ACHIVEMENT_CRASHER
  PokeByte bank, TAchivement.NO/8+1, True
  SaveBank bank, DB_FILE
End Function
Function SaveDatabase%()
  Local bank:TBank = CreateBank(TAchivement.NO/8+2)
  TAchivement.Save bank
  PokeByte bank, TAchivement.NO/8+1, False
  SaveBank bank, DB_FILE
End Function
Function ShowAchivementScreen%()
  Repeat
    WaitTimer timer
    Cls
    achivementsHorn.Draw 0, 0
    achivementsItem1.Draw 0, 150
    achivementsItem2.Draw 0, 300
    achivementsPlayer.Draw 0, 450
    Flip 0
    GCCollect
    If AppTerminate() Then Return 0
  Until KeyHit(KEY_ESCAPE)
  For Local achivement:TAchivement = EachIn TAchivement.array
    achivement.isNew = False
  Next
  Return True
End Function
Function ShowDisplayAchivements()
  Local x# = sx-TAchivement.BOX_WIDTH, y# = sy-TAchivement.BOX_HEIGHT
  For Local achivement:TAchivement = EachIn displayAchivements
    SetColorArray achivement.group.col_new
    If achivement.displayTime < DISPLAY_FADE_TIME Then
      SetAlpha achivement.displayTime/Float(DISPLAY_FADE_TIME)
    Else
      SetAlpha 1.0
    EndIf
    achivement.Draw x, y
    achivement.displayTime :- 1
    If achivement.displayTime <= 0 Then displayAchivements.Remove achivement
    y :- TAchivement.BOX_HEIGHT
    If y <= 0 Then
      y = sy-TAchivement.BOX_HEIGHT
      x :- TAchivement.BOX_WIDTH
    EndIf
  Next
  SetAlpha 1.0
End Function
Function ShowMenu()
  Init
  SetColor 255, 255, 255
  Repeat
    WaitTimer timer
    Cls
    DrawText "Spielen:", 0, 0
    DrawText "einfach", 200, 0
    DrawText "mittel", 400, 0
    DrawText "schwer", 600, 0
    DrawText "Errungenschaften", 0, 20
    If Not alInstalled Then
      DrawText "OpenAL download", 0, 50'connect.creativelabs.com/openal/Downloads/oalinst.zip
    EndIf
    Flip 0
    If MouseHit(MOUSE_LEFT) Then
      If MouseY() < 19 And MouseX() > 150 Then
        Local level%
        If MouseX() < 350 Then
          level = 1
        ElseIf MouseX() < 550 Then
          level = 2
        Else
          level = 3
        EndIf
        CreateEnvironment
        Select Play(level)
         Case 1
          TAchivement.Unlock ACHIVEMENT_WIN
          If MilliSecs()-StartTime < FAST_WIN Then TAchivement.Unlock ACHIVEMENT_WINFAST
         Case -1
          Exit
        End Select
        Reset
      ElseIf MouseY() < 39 Then
        If Not ShowAchivementScreen() Then Exit
      ElseIf Not alInstalled And MouseY() > 49 And MouseY() < 70 Then
        OpenURL "http://connect.creativelabs.com/openal/Downloads/oalinst.zip"
      EndIf
      SetColor 255, 255, 255
    EndIf
    GCCollect
  Until KeyHit(KEY_ESCAPE) Or AppTerminate()
  Terminate
End Function
Function DrawSunset()
  Plot sx/2, 0
  SetColor 255, 255, 0
  glBegin GL_TRIANGLE_FAN
  glVertex2f sx/2,0
  SetColor 255, 50, 75
  glVertex2f 0, sy
  glVertex2f 0, 0
  glVertex2f sx, 0
  glVertex2f sx, sy
  glVertex2f 0, sy
  glEnd
End Function