Global alDevice%, alContext%
Global alBuffer%, hornSample:TAudioSample, alFormat%
Const AUDIO_SCALE# = .01
Global useAL% = True, alInstalled% = OpenALInstalled()
For Local arg$ = EachIn AppArgs
  DebugLog "arg: "+arg
  If "--quiet".Contains(arg) Then
    useAL = False
    DebugLog "quiet"
    Exit
  EndIf
Next
Function OpenALInit()
  If Not useAL Then Return
  If Not alInstalled Then
    useAL = False
    DebugLog "NOTICE: FOR OPTIMAL GAMEPLAY, IT IS RECOMMENTED TO USE OpenAL!"
    Return
  EndIf
  EnableOpenALAudio
  SetAudioDriver "OpenAL"
  alDevice = alcOpenDevice(Null)
  Assert alDevice 
  alContext = alcCreateContext(alDevice , Null)
  Assert alContext
  alcMakeContextCurrent alContext
  CheckAlError
  alListener3f AL_POSITION, 0, 0, 0
  CheckAlError
  alGenBuffers 1, Varptr(alBuffer)
  CheckAlError
End Function
Function LoadHorn()
  If Not useAL Then Return
  hornSample = LoadAudioSample("horn.wav")
  Local factor% = 1
  Select hornSample.format
   Case SF_MONO8
    alFormat = AL_FORMAT_MONO8
    DebugLog "MONO8"
   Case SF_MONO16LE
    alFormat = AL_FORMAT_MONO16
    factor = 2
    ?BigEndian
      hornSample = hornSample.Convert(SF_MONO16BE)
    ?
    DebugLog "MONO16LE"
   Case SF_MONO16BE
    alFormat = AL_FORMAT_MONO16
    factor = 2
    ?LittleEndian
      hornSample = hornSample.Convert(SF_MONO16LE)
    ?
    DebugLog "MONO16BE"
   Case SF_STEREO8
    alFormat = AL_FORMAT_STEREO8
    factor = 2
    DebugLog "STEREO8"
   Case SF_STEREO16LE
    alFormat = AL_FORMAT_STEREO16
    factor = 4
    ?BigEndian
      hornSample = hornSample.Convert(SF_STEREO16BE)
    ?
    DebugLog "STEREO16LE"
   Case SF_STEREO16BE
    alFormat = AL_FORMAT_STEREO16
    factor = 4
    ?LittleEndian
      hornSample = hornSample.Convert(SF_STEREO16LE)
    ?
    DebugLog "STEREO16BE"
   Default
    RuntimeError "Unknown sample format"
  End Select
  alBufferData alBuffer, alFormat, hornSample.samples, hornSample.length*factor, hornSample.hertz
End Function

Function DestroyAL()
  If Not useAL Then Return
  alcDestroyContext alContext
  alcCloseDevice alDevice
  CheckAlError
End Function
Function CheckAlError()
  If Not useAL Then Return
  Local error% = alGetError()
  Select error
   Case AL_INVALID_NAME
    RuntimeError "invalid name"
   Case AL_INVALID_ENUM
    RuntimeError "invalid enum"
   Case AL_INVALID_VALUE
    RuntimeError "invalid value"
   Case AL_INVALID_OPERATION
    RuntimeError "invalid command"
   Case AL_OUT_OF_MEMORY
    RuntimeError "out of memory"
   Case AL_NO_ERROR
   Default
    RuntimeError "unknown error"
  End Select
  error = alcGetError(alDevice)
  Select error
   Case ALC_INVALID_DEVICE
    RuntimeError "invalid device"
   Case ALC_INVALID_CONTEXT
    RuntimeError "invalid context"
   Case ALC_INVALID_ENUM
    RuntimeError "invalid enum"
   Case ALC_INVALID_VALUE
    RuntimeError "invalid value"
   Case ALC_NO_ERROR
   Default
    RuntimeError "unknown error"
  End Select
End Function

Global lastHorn%, b4lastHorn%, noOfHorns% = 0
Const FAST2HORNS% = 6000, FAST3HORNS% = 10000, FAIL_DIST# = 20
Const FAIL_DIST2# = FAIL_DIST * FAIL_DIST
Type THorn Extends TItem
  Field source%, vx#, vy#, speed# = 1.0, near% = False
  Global image:TImage
  Const ANGLE# = 45
  Method New()
    Self.list.AddLast Self
    If useAL Then
      alGenSources 1, Varptr(Self.source)
      alSourcei Self.source, AL_BUFFER, alBuffer
      alSourcei Self.source, AL_LOOPING, AL_TRUE
      alSourcef Self.source, AL_CONE_OUTER_GAIN, Self.ANGLE
    EndIf
    noOfHorns :+ 1
  End Method
  Method Delete()
    Self.Destroy
    DebugLog "Delete Horn"
  End Method
  Method Destroy()
    If Self.source Then
      alDeleteSources 1, Varptr(Self.source)
      Self.source = 0
    EndIf
    Self.list.Remove Self
    noOfHorns :- 1
  End Method
  Method SetPosition(x#, y#)
    Self.x = x
    Self.y = y
    If Not useAL Then Return
    alSource3f Self.source, AL_POSITION, x*AUDIO_SCALE, 0, y*AUDIO_SCALE
  End Method
  Method SetVector(vx#, vy#)
    Self.vx = vx
    Self.vy = vy
    If Not useAL Then Return
    alSource3f Self.source, AL_VELOCITY, vx*AUDIO_SCALE, 0, vy*AUDIO_SCALE
  End Method
  Method Update()
    Self.x :+ Self.vx
    Self.y :+ Self.vx
    If Not useAL Then Return
    alSource3f Self.source, AL_POSITION, Self.x*AUDIO_SCALE, 0, Self.y*AUDIO_SCALE
  End Method
  Method SetSpeed(speed#)
    If Not useAL Then Return
    alSourcef Self.source, AL_PITCH, speed
  End Method
  Method Play()
    If Not useAL Then Return
    alSourcePlay Self.source
  End Method
  Method Pause()
    If Not useAL Then Return
    alSourcePause Self.source
  End Method
  Function Create:THorn(x#, y#, vx#, vy#)
    Local result:THorn = New THorn
    result.SetPosition x, y
    result.SetVector vx, vy
    result.Play
    Return result
  End Function
  Function SetAllSpeed(speed#)
    For Local horn:THorn = EachIn Self.list
      horn.SetSpeed speed
    Next
  End Function
  Function DeleteAll()
    Self.list.Clear
  End Function
  Function PlayAll()
    For Local horn:THorn = EachIn Self.list
      horn.play
    Next
  End Function
  Function PauseAll()
    For Local horn:THorn = EachIn Self.list
      horn.pause
    Next
  End Function
  Method Draw(offsetX#, offsetY#)
    DrawImage Self.image, Self.x+offsetX, Self.y+offsetY
    Local vx# = Self.x-.x, vy# = Self.y-.y
    Local distSquared# = vx*vx + vy*vy
    If distSquared < FAIL_DIST2 Then
      Self.near = True
      If speedFactor > 1.201 Then TAchivement.Unlock ACHIVEMENT_DOPPLER
    ElseIf Self.near Then
      TAchivement.Unlock ACHIVEMENT_HORN_FAIL
    EndIf
  End Method
  Method Collect()
    foundHorns :+ 1
    TAchivement.Unlock ACHIVEMENT_1HORN
    If timeFactor < 1.0 Then TAchivement.Unlock ACHIVEMENT_SLOWHORN
    Local time% = MilliSecs()
    If b4lastHorn And time-b4lastHorn < FAST3HORNS Then
      TAchivement.Unlock ACHIVEMENT_3HORNS_FAST
      TAchivement.Unlock ACHIVEMENT_2HORNS_FAST
    ElseIf lastHorn Then
      If time-lastHorn < FAST2HORNS Then
        TAchivement.Unlock ACHIVEMENT_2HORNS_FAST
      EndIf
      b4lastHorn = lastHorn
    EndIf
    lastHorn = time
    If foundHorns > 1 Then TAchivement.Unlock ACHIVEMENT_2HORN
    If foundHorns > 2 Then TAchivement.Unlock ACHIVEMENT_3HORN
    Self.Destroy
    If foundHorns >= START_HORNS Then infoTime = START_INFO_TIME
  End Method
  Function DrawAll(offsetX#, offsetY#)
    For Local horn:THorn = EachIn Self.list
      horn.Draw offsetX, offsetY
    Next
  End Function
End Type
Function SetListenerPos(x#, y#)
  If Not useAL Then Return

  alListener3f AL_POSITION, x*AUDIO_SCALE, 0, y*AUDIO_SCALE
End Function
Function SetListenerVector(vx#, vy#)
  If Not useAL Then Return

  alListener3f AL_VELOCITY, vx*AUDIO_SCALE, 0, vy*AUDIO_SCALE
End Function
Function SetListenerDir(dir#)
  If Not useAL Then Return

  Local angles#[] = [Float(Sin(dir)), 0.0, Float(Cos(dir)), 0.0, 1.0, 0.0]
  alListenerfv AL_ORIENTATION, angles
End Function